/*
	ArtoTI.cpp - Library for communicating between an Arduino and 
	TI 83+, 84+, or 86
	Created by William Massano, February 14, 2014
	Released to the public domain after magazine publication
*/
#include <Arduino.h>
#include <ArtoTI.h>

ArtoTI::ArtoTI(byte tippin,byte ringpin){					  //constructor
	Start(tippin,ringpin);
}

void ArtoTI::clearlines(){            //clean out any leftovers from previous sessions
//essentially getByte
  unsigned long timer; 

  resetlines();
  timer=millis();                      //begin timing
  while ((millis() -timer)<500){    
    if (digitalRead(Data0)==LOW){     //it's a zero
        pinMode(Data1,OUTPUT);
        digitalWrite(Data1,LOW);      //acknowledge on Data1;
       
        while (digitalRead(Data0)==LOW); //wait for Data0 to go high

        pinMode(Data1,INPUT_PULLUP); //idle Data1 again
        goto bitDone;                //factor in a zero
        timer=millis();              //reset timer
      } 
      
      if(digitalRead(Data1)==LOW){      //it's a one
        pinMode(Data0,OUTPUT);         
        digitalWrite(Data0,LOW);      //acknowledge on Data0

        while (digitalRead(Data1)==LOW); //wait for Data1 to go high
        
        pinMode(Data0,INPUT_PULLUP);   //idle Data0 again
        
        timer=millis();                //reset timer
        goto bitDone;
      }   
bitDone:   ;                         //ready for next bit
  }                                  // we're still idling
    
  resetlines();
}
    
void ArtoTI::clearScreen(){
  if(comp==0x06) sendCmd(0x008);
  else if (comp==0x23) sendCmd(0x0009);               //clear
}

void ArtoTI::Enter(){          // send appropriate enter command
  if(comp==0x23) sendCmd(0x0005);
  if(comp==0x06) sendCmd(0x0006);
}

void ArtoTI::floatout(double num,int dplaces){
//print floating point number in E format if necessary
//this fixes a flaw in Serial.print
  //we will assume anything> 10^8 or smaller than 10^8 will be printed as E format]
  double temp,posnum;
  boolean negsign;
  int pwr;
  
  if(num<0){
    negsign=true;
    posnum=-num;
  }else{
    negsign=false;
    posnum=num;
  }
  //do big numbers here
  if(posnum==0.0) Serial.print(num,dplaces);
  else if((posnum>1.0e8)||(posnum<1.0e-8)||(posnum<pow(10,-dplaces))){
    temp=log10(posnum);
    pwr=(int)temp;          // the cast is belt and suspenders
    if (pwr<0) pwr=pwr-1;   //gives 1 digit before dp
    temp=temp-pwr;          //get mantissa
    temp=pow(10,temp);      //back to a number
    if(negsign) Serial.print("-");
    Serial.print(temp,dplaces);
    Serial.print(" E");
    Serial.print(pwr);
  } else Serial.print(num,dplaces);
}

byte ArtoTI::getByte(){                   //get a single byte return in function name to paramB
  byte paramB,i;                        //make it local
  
  resetlines();
  paramB = 0;                          //byte built up here
  for(i=0;i<8;i++){    //eight bits to a byte
    paramB = paramB / 2;               //make room for next bit
    while (true) {                    //both lines idle high
    if (digitalRead(Data0)==LOW){     //it's a zero
        pinMode(Data1,OUTPUT);
        digitalWrite(Data1,LOW);      //acknowledge on Data1;
       
        while (digitalRead(Data0)==LOW); //wait for Data0 to go high

        pinMode(Data1,INPUT_PULLUP); //idle Data1 again
        goto bitDone;                //factor in a zero
      } 
      
      if(digitalRead(Data1)==LOW){      //it's a one
        pinMode(Data0,OUTPUT);         
        digitalWrite(Data0,LOW);      //acknowledge on Data0

        while (digitalRead(Data1)==LOW); //wait for Data1 to go high
        
        pinMode(Data0,INPUT_PULLUP);   //idle Data0 again
        
        bitSet(paramB,7);
        goto bitDone;
      }   
    }                                  // we're still idling
    
bitDone:   ;                         //ready for next bit
  }
  return paramB;
  resetlines();
}

boolean ArtoTI::getByteTimed(byte *val){                   //get a single byte return in function name to paramB
//essentially getByte with a timeout ability
//used for getStatus
  byte paramB,i;                        //make it local
  long timer;
  
  resetlines();
  paramB = 0;                          //byte built up here
  for(i=0;i<8;i++){    //eight bits to a byte
    paramB = paramB / 2;               //make room for next bit
    timer=millis();
    while ((millis()-timer)<200) {                    //both lines idle high
    if (digitalRead(Data0)==LOW){     //it's a zero
        pinMode(Data1,OUTPUT);
        digitalWrite(Data1,LOW);      //acknowledge on Data1;
       
        while (digitalRead(Data0)==LOW); //wait for Data0 to go high

        pinMode(Data1,INPUT_PULLUP); //idle Data1 again
        goto bitDone;                //factor in a zero
      } 
      
      if(digitalRead(Data1)==LOW){      //it's a one
        pinMode(Data0,OUTPUT);         
        digitalWrite(Data0,LOW);      //acknowledge on Data0

        while (digitalRead(Data1)==LOW); //wait for Data1 to go high
        
        pinMode(Data0,INPUT_PULLUP);   //idle Data0 again
        
        bitSet(paramB,7);
        goto bitDone;
      }   
    }                                  // we're still idling
  goto timedout;
  bitDone:   ;                         //ready for next bit
  }
  resetlines();  
  *val=paramB;
  return true;
timedout: paramB=0;
  return false;  
}

void ArtoTI::getDir(){
  //get and print a directory of variables in calctype 
  //mostly to test silent i/o
  
  byte inbuf[20];
  byte i,datalength;
  word checksum;
  word availRAM;
  boolean done;
  char q;
  
  if(comp==0x06){
    getDir86();
    return;
  }
  //send dirctory request
  sendByte(comp);                  // who is sending
  sendByte(0xA2);                  //directory request
  sendByte(0x0D);                  //header has 13 bytes not in checksum
  sendByte(0x00);                  //first half of above litte endian
  sendByte(0x00);                  //length of data field (0 but padded)
  sendByte(0x00);                  //above is 16 bits
  sendByte(0x19);                  //the actualy request
  for(i=0;i<10;i++) sendByte(0x00);  // 10 bytes of padding
  sendByte(0x19);                  //lower byte of checksum
  sendByte(0x00);                  //upper byte of checksum
  for(i=0;i<4;i++){                //get ACK
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=ACK){
    Serial.println("ACK not received in getDir86");
    return;
  }
  //got ack, get free memory
  for (i=0;i<8;i++){
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=DATA){
    Serial.println("Data packet not received in getDir");
    return;
  }
  //compute checksum
  checksum=inbuf[4]+inbuf[5];
  if (checksum!=(inbuf[7]*256+inbuf[6])){    //little endian!
    Serial.println("checksums do not match in getDir");
    return;
  }
  sendACK();              //acknowledge
  availRAM=256*inbuf[5]+inbuf[4];
  Serial.print("Free memory in calculator= ");
  Serial.println(availRAM);
  done=false;
  while(!done){
    //get next 4 bytes
    for(i=0;i<4;i++){
      inbuf[i]=0;
      inbuf[i]=getByte();
    }
    //are we at end?
    if(inbuf[1]==EOT){            //we're done
      sendACK();
      done=true;                   //bye
    } else {
      if(inbuf[1]!=VHEADER){
        Serial.println("variable header not received in getDir");
        return;
      }
      datalength=256*inbuf[3]+inbuf[2];
      //get data
      for(i=0;i<datalength+2;i++){      //get checksum too
        inbuf[i]=0;
        inbuf[i]=getByte();
      }
/*      Serial.println("inbuf[2]  inbuf[3]");
      Serial.print(inbuf[2],HEX);
      Serial.print("  ");
      Serial.println(inbuf[3],HEX);
*/
      if(inbuf[2]==0){            //just print the A-Z variable if they exist
        Serial.print("Variable name= ");
        q=inbuf[3];
        Serial.print(q);
        if(inbuf[12]==0) Serial.println();
        else Serial.println(" archived");
      }
      sendACK();
    }
  }    
}

void ArtoTI::getDir86(){
  //get and print a directory of variables in calctype 
  //mostly to test silent i/o
  
  byte inbuf[20];
  byte i,j,datalength;
  word checksum;
  long availRAM;
  boolean done;
  char q;
  
  //send dirctory request
  sendByte(comp);                  // who is sending
  sendByte(0xA2);                  //directory request
  sendByte(0x05);                  //header has 13 bytes not in checksum
  sendByte(0x00);                  //first half of above litte endian
  sendByte(0x00);                  //length of data field (0 but padded)
  sendByte(0x00);                  //above is 16 bits
  sendByte(0x15);                  //the actualy request
  sendByte(0x00);                  //namelength
  sendByte(0x00);                  //name[0]
  sendByte(0x15);                  //lower byte of checksum
  sendByte(0x00);                  //upper byte of checksum
  for(i=0;i<4;i++){                //get ACK
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=ACK){
    Serial.println("ACK not received in getDir86");
    return;
  }
  //got ack, get free memory
  for (i=0;i<9;i++){
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=DATA){
    Serial.println("Data packet not received in getDir86");
    return;
  }
  //compute checksum
  checksum=inbuf[4]+inbuf[5]+inbuf[6];
  if (checksum!=(inbuf[8]*256+inbuf[7])){    //little endian!
    Serial.println("checksums do not match in getDir86");
    return;
  }
  sendACK();              //acknowledge
/*  for(i=0;i<9;i++){
    Serial.print(inbuf[i],HEX);
    Serial.print("  ");   
  }
  Serial.println();*/ 
  availRAM=65536*inbuf[4]+inbuf[5]+inbuf[6]*256;
  Serial.print("Free memory in calculator= ");
  Serial.println(availRAM);
  done=false;
  Serial.println("Variable names");
  while(!done){
    //get next 4 bytes
    for(i=0;i<4;i++){
      inbuf[i]=0;
      inbuf[i]=getByte();
    }
    //are we at end?
    if(inbuf[1]==EOT){            //we're done
      sendACK();
      done=true;                   //bye
    } else {
      if(inbuf[1]!=VHEADER){
        Serial.println("variable header not received in getDir86");
        return;
      }
      datalength=256*inbuf[3]+inbuf[2];
      //get data
      for(i=0;i<datalength+2;i++){      //get checksum too
        inbuf[i]=0;
        inbuf[i]=getByte();
/*        Serial.print(inbuf[i],HEX);
        Serial.print("  "); */
      }
//      Serial.println();
      for(j=4;j<(4+inbuf[3]);j++){
        q=inbuf[j];
        Serial.print(q);
      }
      Serial.println();

      sendACK();
    }
  }    
}  //getDir86

double ArtoTI::getNum(char *sname){
  //INPUT: nam = real number variable name (A-Z)
  byte inbuf[30],dummyB,nam;
  int i,j;
  byte numTI[10];                          //packed value in TI format
  word chkW;
  char val[30];
  double temp;
  double sign;
  double exponent;
  
  if(comp==0x06) return getNum86(sname);  //it's an 86
  nam=sname[0];
  for (i=0;i<10;i++) numTI[i]=0;
  for(i=0;i<30;i++) val[i]=0;         // initialize val
  sendByte(comp);                     //machine ID 
  sendByte(0xA2);                     //REQ
  sendByte(0x0D);                     //length of command-LSB
  sendByte(0x00);                     //length of command-MSB
  
  sendByte(0x09);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  sendByte(0x00);                     //ID for real number
  sendByte(nam);                     //ASCII of name name (A-Z)

  for(i=0;i<9;i++) sendByte(0x00);    //padding and dud bytes

  chkW = (word) nam + 9;             //create checksum
  sendByte(lowByte(chkW));            //send LSB
  sendByte(highByte(chkW));           //send MSB
  
  for(i=0;i<23;i++){  
    inbuf[i]=0;
    inbuf[i]=getByte();                //get response from TI
  }
  if(inbuf[1]!=ACK){
      Serial.println("ACK for variable request not sent");
      temp=0.0;
      goto getNum1;
  }
  //rest of ACK in ignored
  sendACK();
  
  sendByte(comp);                     //machine ID
  sendByte(0x09);                     //CTS  
  sendByte(0x00);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB

  for(i=0;i<4;i++){  
    inbuf[i]=0;
    inbuf[i]=getByte();                //get response from TI
  }
  if(inbuf[1]!=ACK){
      Serial.println("ACK for CTS not sent");
      temp=0.0;
      goto getNum1;
  }
  
  for(i=0;i<15;i++){                  //get data from TI
    inbuf[i]=0;
    inbuf[i]=getByte();
  }

  sendACK();

  for(i=0;i<11;i++){
    numTI[i]=0;
    numTI[i]=inbuf[i+4];
  }
  
//convert numTI to a double
  //byte 0 sign
  if (numTI[0]==0x0080) sign=-1.0;
  else sign=1;
  exponent=numTI[1]-128;              //128=80h and is the offset
  //byte 2-8 are the mantissa
  for(j=0;j<30;j++) val[j]=0;         //init val
  j=0;
  //first numTI gets special treatment
  val[j]=(numTI[2]/16)+ASCII0;        //high byte
  j++;
  val[j]='.';
  j++;
  val[j]=(numTI[2] & 0x0F)+ASCII0;
  j++;
  //now the rest
  for(i=3;i<9;i++){
    val[j]=(numTI[i]/16)+ASCII0;        //high byte
    j++;
    val[j]=(numTI[i] & 0x0F)+ASCII0;
    j++;
  }                //val should still have a null at the end
  Serial.print("Val in getNum: ");
  Serial.println(val);
  temp=strtod(val,NULL);
  temp=temp*sign*pow(10,exponent);
getNum1: delay(100); 
  return temp;
} //end getNum 

double ArtoTI::getNum86(char *sname){
  //INPUT: nam = real number variable name (A-Z)
  byte inbuf[30],dummyB,nam,namelength;
  int i,j;
  byte numTI[10];                          //packed value in TI format
  word chkW;
  char val[30];
  double temp;
  double sign;
  double exponent;

  namelength=len(sname);  
  for (i=0;i<10;i++) numTI[i]=0;
  for(i=0;i<30;i++) val[i]=0;         // initialize val
  sendByte(comp);                     //machine ID 
  sendByte(0xA2);                     //REQ
  sendByte(0x0C);                     //length of command-LSB
  sendByte(0x00);                     //length of command-MSB
  
  sendByte(0x0A);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  sendByte(0x00);                     //ID for real number
  sendByte(namelength);               //length of nale
  for(i=0;i<namelength;i++){
  sendByte(sname[i]);                     //ASCII of name name (A-Z)
  }
  
  for(i=0;i<(8-namelength);i++) sendByte(0x20);    //padding and dud bytes

  chkW = (word) namelength+0x000A;             //create checksum
  for(i=0;i<namelength;i++)chkW=chkW+(word)sname[i];
  for(i=0;i<(8-namelength);i++) chkW=chkW+0x0020;
  
  sendByte(lowByte(chkW));            //send LSB
  sendByte(highByte(chkW));           //send MSB
  for(i=0;i<4;i++){  
    inbuf[i]=0;
    inbuf[i]=getByte();                //get ACKresponse from TI
  }
  if(inbuf[1]!=ACK){
      Serial.println("ACK for variable request not sent");
      temp=0.0;
      goto getNum1;
  }
  //get first four bytes of second response
  for(i=0;i<4;i++){
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  j=inbuf[2]+2;                  //length of rest of packet
  for(i=4;i<(j+4);i++)inbuf[i]=getByte();   //just ignore for now
  
  sendACK();
  sendByte(comp);                     //machine ID
  sendByte(0x09);                     //CTS  
  sendByte(0x00);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB

  for(i=0;i<4;i++){  
    inbuf[i]=0;
    inbuf[i]=getByte();              //get response from TI
  }
  if(inbuf[1]!=ACK){
      Serial.println("ACK for CTS not sent");
      temp=0.0;
      goto getNum1;
  }
  
  for(i=0;i<4;i++){                  //get data from TI
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  j=inbuf[2]+2;                      //data + checksum bytes
  for(i=0;i<j;i++){
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  sendACK();

  for(i=0;i<(j-2);i++){                //datalength less 2 for cksum
    numTI[i]=0;
    numTI[i]=inbuf[i];
  }
  
//convert numTI to a double
  //byte 0 sign
  if (numTI[0]==0x0080) sign=-1.0;
  else sign=1;
  exponent=numTI[1]+numTI[2]*256-0xFC00;              //FC00h and is the offset
  //byte 3-10 are the mantissa
  for(j=0;j<30;j++) val[j]=0;         //init val
  j=0;
  //first numTI gets special treatment
  val[j]=(numTI[3]/16)+ASCII0;        //high byte
  j++;
  val[j]='.';
  j++;
  val[j]=(numTI[3] & 0x0F)+ASCII0;
  j++;
  //now the rest
  for(i=4;i<10;i++){
    val[j]=(numTI[i]/16)+ASCII0;        //high byte
    j++;
    val[j]=(numTI[i] & 0x0F)+ASCII0;
    j++;
  }                //val should still have a null at the end
  temp=strtod(val,NULL);
  temp=temp*sign*pow(10,exponent);
getNum1: delay(100); 
  return temp;
} //end getNum 86

byte ArtoTI::getStatus(){
  /* since all calculators will respond to an 82 command,
  we will start with that and see who responds later we may have to
  use the timeout feature*/
  //it works, but is fairly useless as 82's don't allow silent xfer
  //I'll leave it here for future reference
  byte i;
  byte inbuf[4];
  boolean foundit;
  
  //send get status command
  //try 82,83,83+,84+ first (all respond to 82)
  sendByte(0x02);            //pretend it is an 82
  sendByte(0x68);
  sendByte(0x00);
  sendByte(0x00);
  foundit=true;
  i=0;
  while (foundit && (i<4)){
    inbuf[i]=0;
    foundit=getByteTimed(&inbuf[i]);
    if(foundit) i++;
  }
  if(!foundit){              //maybe it's an 86
    sendByte(0x06);          //try an 86
    sendByte(0x68);
    sendByte(0x00);
    sendByte(0x00);
    foundit=true;
    i=0;
    while (foundit && (i<4)){
      inbuf[i]=0;
      foundit=getByteTimed(&inbuf[i]);
      if(foundit)i++;
    }
  }
  if(!foundit) return 0;      //no id  
  for(i=0;i<4;i++){
    Serial.print(inbuf[i], HEX);
    Serial.print(" ");
  }
  Serial.println();
  switch (inbuf[0]){
    case 0x73:  comp=0x23;    //TI 83+/84+
                if(inbuf[3]==0x00) {
                  Serial.println("TI 83+ Connected");
                  calctype=83;
                }else if(inbuf[3]==0x08) {
                  Serial.println("TI84+ connected");
                  calctype=84;
                }
                break;
    case 0x82:  comp=0x02;      //computer sending to TI88
                Serial.println("TI 82 Connected");
                calctype=82;
                break;
    case 0x83:  comp=0x03;      //computer sending to TI83
                Serial.println("TI 83 Connected");
                calctype=-83;
                break;
    case 0x86:  comp=0x06;      //computer sending to TI83
                Serial.println("TI 86 Connected");
                calctype=86;
                break;
  }
  return comp;    //give it a nonzero return value
}

byte ArtoTI::len(char *str){
  /*rather than convert to arduino string class, I'll make my own 
  string length function
  report length less the null 
 and yes I know this is just strlen */
  byte counter;
  counter=0;
  while (str[counter]!='\0') counter++;
  return counter;
}    

char* ArtoTI::ltrim(char *string, char junk){
  char* original = string;
  char *p = original;
  int trimmed = 0;
  do{
    if (*original != junk || trimmed){
      trimmed = 1;
      *p++ = *original;
    }
  }while (*original++ != '\0');
  return string;
}

void ArtoTI::numtoTIst83(double num, byte *numTI){  //numTI will have 10 elements
  int i,j;
  double mantissa,temp;          //to get he mantissa
  char value[20],stripped[20];
  byte tempb;
  
  for(i=0;i<10;i++) numTI[i]=0;  //init numTI
  //byte 0 sign
  if(num<0) numTI[0]=0x0080;
  else numTI[0]=0;
  //byte 1 the exponent offset by 80h=128
  numTI[1]=128+(int)log10(abs(num));  //83/84+
#ifdef DEBUG
  Serial.print("num=");
  Serial.println(num);
  Serial.print("numTI[1]=");
  Serial.println(numTI[1]);
#endif
  //remaining 7 bytes hold mantissa -14digits, dp after first
  //get mantissa
  temp=abs(num);
  temp=log10(temp)-(int)log10(temp);   //get mantissa
  mantissa=pow(10,temp);
  //now convert mantissa to string for waqsy parsing
  for(i=0;i<20;i++) value[i]=0;         //init value
  dtostrf(mantissa,16,13,value);        //convert num to a string
/*value should have blank (since mantissa is +) 0-9 or blank for
  first digit and up to 13 more decimal digits
  */
  ltrim(value,' ');                     //kill blanks
  //zero stripped
  for(j=0;j<20;j++)stripped[j]=0;
  j=0;                                  //point to value in stripped 
  for(i=0;i<20;i++){                    //remove decimal point
    if(value[i]!='.'){
      stripped[j]=value[i];
      j++;
    }
  }
  //now pack numTI 2-9 with the bcd characters in stripped
  j=0;                                  //stripped index
  for(i=2;i<10;i++){                    //each numTI holds 2 digits
    if(stripped[j]==0){
      tempb=0;
      j++;
    }else{
      tempb=stripped[j]-ASCII0;           
      j++;
    }
    numTI[i]=16*tempb;                   //high nibble
    if(stripped[j]==0){
      tempb=0;
      j++;
    }else{
      tempb=stripped[j]-ASCII0;           
      j++;
    }
    numTI[i]=numTI[i]+tempb;              //low nibble
  }    
#ifdef DEBUG
  for(i=0;i<10;i++){
    Serial.print(numTI[i],HEX);
    Serial.print(",");
  }
  Serial.println();
#endif
}

void ArtoTI::numtoTIst86(double num, byte *numTI){  //numTI will have 10 elements
  int i,j;
  double mantissa,temp;          //to get he mantissa
  char value[20],stripped[20];
  byte tempb;
  word paramW;
  
  for(i=0;i<10;i++) numTI[i]=0;  //init numTI
  //byte 0 sign
  if(num<0) numTI[0]=0x0080;
  else numTI[0]=0;
  //byte 1&2 the exponent offset by FC00h
  paramW=0xFC00+(int)log10(abs(num));
  //little endian
  numTI[1]=lowByte(paramW);
  numTI[2]=highByte(paramW);
  //remaining 7 bytes hold mantissa -14digits, dp after first
  //get mantissa
  temp=abs(num);
  temp=log10(temp)-(int)log10(temp);   //get mantissa
  mantissa=pow(10,temp);
  //now convert mantissa to string for waqsy parsing
  for(i=0;i<20;i++) value[i]=0;         //init value
  dtostrf(mantissa,16,13,value);        //convert num to a string
/*value should have blank (since mantissa is +) 0-9 or blank for
  first digit and up to 13 more decimal digits
  */
  ltrim(value,' ');                     //kill blanks
  //zero stripped
  for(j=0;j<20;j++)stripped[j]=0;
  j=0;                                  //point to value in stripped 
  for(i=0;i<20;i++){                    //remove decimal point
    if(value[i]!='.'){
      stripped[j]=value[i];
      j++;
    }
  }
  //now pack numTI 2-9 with the bcd characters in stripped
  j=0;                                  //stripped index
  for(i=3;i<10;i++){                    //each numTI holds 2 digits
    if(stripped[j]==0){
      tempb=0;
      j++;
    }else{
      tempb=stripped[j]-ASCII0;           
      j++;
    }
    numTI[i]=16*tempb;                   //high nibble
    if(stripped[j]==0){
      tempb=0;
      j++;
    }else{
      tempb=stripped[j]-ASCII0;           
      j++;
    }
    numTI[i]=numTI[i]+tempb;              //low nibble
  }    
}

void ArtoTI::Plot(double *xlist, double *ylist, int numpoints, byte mark){
  //plot y vs x on 83+/84+
  //mark is 0 - box, 1 - plus, 2 - point
  byte i;
  
  if(comp==0x06) return Plot86(xlist,ylist,numpoints,mark);
  sendCmd(0x36);  //clear lists
  sendANstring("4");
  Enter();
#ifdef DEBUG
  Serial.println("Sending 2 10-element lists");
#endif
  sendList(1,xlist,numpoints);  //list 1 , 18 elements
  sendList(2,ylist,numpoints);
  sendCmd(0x55);      //statplot
  Enter();
 //what tic mark box, + or .
  if(mark<3){
    sendANstring("%DN%DN%DN%DN");  //4 down arrows
    for(i=1;i<=mark;i++){
      sendANstring("%RT");
    }
    Enter();
  }
  
  sendCmd(0x68);  //stat zoom and plot
}


void ArtoTI::Plot86(double *xlist, double *ylist, int numpoints, byte mark){
  //plot y vs x on 86
  //mark is 0 - box, 1 - plus, 2 - point
  byte i;
  
#ifdef DEBUG
  Serial.println("Sending 2 10-element lists");
#endif
  sendList(1,xlist,numpoints);  //list 1 , 18 elements
  sendList(2,ylist,numpoints);
  sendCmd(0x81);    //graph menu
  //clear all y=functions
  sendCmd(0xC2);  //F1 y=
  sendCmd(0x0B);  //more
  sendCmd(0xC3);  //F2 delete all
  sendCmd(0x80);  //stat cmd
  sendCmd(0xC4);  //F3 plot
  sendCmd(0xC2);  //F1 plot1
  sendANstring("%LF");  //left over on
  Enter();
  sendANstring("%DN%DN%DN%DN");  //down to marks
  switch (mark){
    case 0:  sendCmd(0xC2);  //box
             break;
    case 1:  sendCmd(0xC3);  //+
             break;
    case 2:  sendCmd(0xC4);  //.
             break;
    default: sendCmd(0xC2);
  }
  sendCmd(0x81);  //graph
  sendCmd(0xC4);  //Zoom
  sendCmd(0x0B);  //more
  sendCmd(0xC6);  //zdata
}


void ArtoTI::resetlines(){
//set both pins to INPUT_PULLUP mode
  pinMode(Data0,INPUT_PULLUP);
  pinMode(Data1,INPUT_PULLUP);
}

void ArtoTI::sendACK(){
  sendByte(comp);                     //machine ID
  sendByte(0x56);                     //ACK
  sendByte(0x00);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
}

void ArtoTI::sendANkey(char key){
  //send characters as keystrokes
  word outkey;
  
  switch (comp){
    case 0x06:  //86
                if((key>0x40) && (key<0x5B)){
                  //A-Z
                  outkey=key-0x19;
                }else if((key>0x60)&&(key<0x7B)){
                  //a-z
                  outkey=key-0x1F;
                }else if((key>0x2F)&&(key<0x3A)){
                  //0-9
                  outkey=key-0x14;
                }else if(key==' ') outkey=0x27;
                else outkey=0;
                if(outkey!=0) sendCmd(outkey);
                break;
     case 0x23:  //83+/84 
                if((key>0x40) && (key<0x5B)){
                  //A-Z
                  outkey=key+0x59;
                }else if((key>0x60)&&(key<0x7B)){
                  //a-z
                  outkey=key-'a'+0xFCE2;
                }else if((key>0x2F)&&(key<0x3A)){
                  //0-9
                  outkey=key+0x5E;
                }else if(key==' ') outkey=0x99;
                else outkey=0;
                if(outkey!=0) sendCmd(outkey);
                break;
  }  
}//sendANkey

void ArtoTI::sendANstring(char *str){
  //I have no desire to write a full parser so some ops must be sent with sendOP
  //so much for above, I need to make this parser smarter
  //I will now allow a quote character the percent sign to allow non AN operators
  byte i;
  byte caretcounter;    //to handle 84
  char q;    //temp character
  char temp[3];
  
  caretcounter=0;
  for(i=0;i<len(str);i++){
    q=str[i];
    strcpy(temp,"");
    temp[0]=q;
#ifdef DEBUG
//    Serial.print("q=");
//    Serial.println(q);
#endif
    switch (q){
      case '{':  sendOP(temp);
                 break;
      case '}':  sendOP(temp);
                 break;
      case '(':  sendOP(temp);
                 break;
      case ',':  sendOP(temp);
                 break;
      case ')':  if(caretcounter>0){
                    sendCmd(0x0001);  //send down arrow
                    caretcounter--;
                 }
				 sendOP(temp);
                 break;
      case '+':  if(caretcounter>0){
                    sendCmd(0x0001);  //send right arrow
                    caretcounter--;
                  }
                  sendOP(temp);
                  break;
      case '-':  if(caretcounter>0){
                    sendCmd(0x0001);  //send down arrow
                    caretcounter--;
                  }
                  sendOP(temp);
                  break;
      case '/':  if(caretcounter>0){
                    sendCmd(0x0001);  //send down arrow
                    caretcounter--;
                  }
                  sendOP(temp);
                  break;
      case '*':  if(caretcounter>0){
                    sendCmd(0x0001);  //send down arrow
                    caretcounter--;
                  }
                  sendOP(temp);
                  break;
      case '^':  if(calctype==84) caretcounter++;
                 sendOP(temp);
                 break;
      case '.': sendOP(temp);
                break;
      case '%': if(str[i+1]=='%'){//got quote character
                  //just send a %
                  sendOP("%");
                  i++;        //incremnet i to kill second %
                } else {
                  temp[0]=str[i+1];
                  temp[1]=str[i+2];
                  temp[2]=0;
                  i=i+2;  //skip op string
                  sendOP(temp);
                  strcpy(temp,"");
                  q=0;
                }
                break;  
      default:  if(q!=' '){
                  if(q<'0') return;
                  if((q>'9')&&(q<'A'))return;
                  if(q>'z')return;  
                  sendANkey(q);
      }
    }
  }
  return;
}

void ArtoTI::sendByte(byte paramB){          //send a single byte
byte i;
  for(i=0;i<8;i++){                   //eight bits in byte
    if (bitRead(paramB,0)==0) {        //fetch next bit
      pinMode(Data0,OUTPUT);          //alert calculator
      digitalWrite(Data0,LOW);           

      while (digitalRead(Data1)==HIGH);  //wait for acknowledgment

      digitalWrite(Data0,HIGH);       //acknowledge acknowledgment

      while (digitalRead(Data1)==LOW);  //wait for all clear

    }else{     
      pinMode(Data1,OUTPUT);           //alert calculator
      digitalWrite(Data1,LOW);           

      while (digitalRead(Data0)==HIGH);  //wait for acknowledgment
      
      digitalWrite(Data1,HIGH);          //acknowledge acknowledgment

      while (digitalRead(Data0)==LOW);   //wait for all clear

    }

    paramB = paramB / 2;                //get set for next bit
    resetlines();                       //idle the lines again
  }
}

void ArtoTI::sendCmd(word paramW){             //send a 16-bit key command
  byte inbuf[4];
  byte i;
  sendByte(comp);                      //machine ID
  sendByte(0x87);                      //CMD
  sendByte(lowByte(paramW));           //command-LSB
  sendByte(highByte(paramW));          //command-MSB
  
  for(i=0;i<4;i++){                    //dump ACK from TI
    inbuf[i]=0;
    inbuf[i]=getByte();                //get first ACK
  }
  if(inbuf[1]==ACK){
      for(i=0;i<4;i++){                  //got first ACK from TI
        inbuf[i]=0;
        inbuf[i]=getByte();                //get second ACK
      }
      if(inbuf[1]!=ACK)Serial.println("Didn't get second ACK in getByte");
  }else Serial.println("Didn't get first ACK in getByte");
}  

void ArtoTI::sendList(byte listnum,double *list,int listlen){
/*Since silent xfer isn't working for lists, we'll do it this
way. it's kludgy, but it should work*/
  char temps[20];      //for string to be sent
  byte i,j;
  word tl;
  
  if(comp==0x06) {
    sendList86(listnum, list,listlen);
    return;  
  }
  sendANstring("{");  //leading brace
  i=0;
  while(i<listlen){
    for(j=0;j<20;j++)temps[j]=0;
    dtostrf(list[i],14,5,temps);
#ifdef DEBUG
    Serial.print("List[");
    Serial.print(i);
    Serial.print("]=");
    Serial.print(list[i]);
    Serial.print("  temps=");
    Serial.println(ltrim(temps,' '));
#endif
    sendANstring(ltrim(temps,' '));
    i++;
    if(i<listlen)sendANstring(",");      
  }
  //close and store
  sendANstring("}%ST");
  tl=0xF2+listnum;
  sendCmd(tl);
  Enter();  
}

void ArtoTI::sendList86(byte listnum, double *list,int listlen){
  char temps[20];      //for string to be sent
  byte i,j;
  word tl;

  sendCmd(0x7A);            //list
  sendCmd(0xC2);            //F1 {
  i=0;
  while(i<listlen){
    for(j=0;j<20;j++)temps[j]=0;
    dtostrf(list[i],14,5,temps);
#ifdef DEBUG
    Serial.print("List[");
    Serial.print(i);
    Serial.print("]=");
    Serial.print(list[i]);
    Serial.print("  temps=");
    Serial.println(ltrim(temps,' '));
#endif
    sendANstring(ltrim(temps,' '));
    i++;
    if(i<listlen)sendANstring(",");      
  }
  
  sendCmd(0xC3);            //F2 }
  sendANstring("%ST");
  sendCmd(0xC4);            //F3 names
  if(listnum==1){
    sendCmd(0xC3);            //F2 xStat
  }else if(listnum==2){
    sendCmd(0xC4);            //F3 yStat
  }
  Enter();
  sendCmd(0x07);            //exit
  sendCmd(0x07);            //exit
}

boolean ArtoTI::sendNum(char *sname,double num){
/* send a number to calculator in TI protocol
*/
  boolean worked=false;          //did xfer work?
  byte i,j,tempb,nam;
  word chkW,paramW;
  boolean errflag;
  byte inbuf[10];                //hold responses
  byte numTI[10];                //data in ti format
  double mantissa,temp;          //to get he mantissa
  char value[20],stripped[20];

  if(comp==0x06) return sendNum86(sname,num);
  nam=sname[0];
  numtoTIst83(num,numTI);
  //now to send
  worked=false;   
  errflag=false;                      //in case we err
#ifdef DEBUG
  Serial.println("Sending");
#endif
  sendByte(comp);                     //machine ID 
  sendByte(0xC9);                     //RTS
  sendByte(0x0D);                     //length of command-LSB
  sendByte(0x00);                     //length of command-MSB
  sendByte(0x09);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  sendByte(0x00);                     //ID for real number
  sendByte(nam);                     //ASCII of variable name (A-Z)

  for(i=0;i<9;i++) sendByte(0x00);    //padding and dud bytes
 
  chkW = (word) nam + 9;             //create checksum
  sendByte(lowByte(chkW));            //send LSB
  sendByte(highByte(chkW));           //send MSB
  delay(100);
  for(i=0;i<4;i++) {                  //get response
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=ACK){
#ifdef DEBUG
    for(i=0;i<4;i++){
      Serial.print(inbuf[i]);
      Serial.print(" ");
    }
    Serial.println(" received. ACK not received");
#endif
    errflag=true;
    goto err;
  }
  for(i=0;i<4;i++) {                  //get response
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=CTS){
#ifdef DEBUG
    Serial.println("CTS not received");
#endif
    errflag=true;
    goto err;
  }
   
#ifdef DEBUG
  Serial.println("Sending ACK");
#endif
  delay(100);
  sendACK();
  sendByte(comp);                     //machine ID
  sendByte(0x15);                     //DATA  
  sendByte(0x09);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB

  chkW = 0;                           //send the value and
  for (j = 0;j<9;j++){       //create checksum simultaneously
    sendByte(numTI[j]);
    chkW = chkW + numTI[j];  
  }

  sendByte(lowByte(chkW));            //send checksum in parts
  sendByte(highByte(chkW));

  for(i=0;i<4;i++) {
    inbuf[i]=0;
    inbuf[i]=getByte();               //response from TI
  }
    if(inbuf[1]!=ACK){
#ifdef DEBUG
      Serial.println("ACK not received");
#endif
      errflag=true;
      goto err;
  }

  sendByte(comp);                     //machine ID
  sendByte(0x92);                     //EOT
  sendByte(0x00);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  
err: worked=!errflag;
  delay(100);                        //give calc time to recover
  return worked;
}

boolean ArtoTI::sendNum86(char *sname,double num){
/* send a number to calculator in TI protocol
*/
  boolean worked=false;          //did xfer work?
  byte i,j,tempb,nam;
  word chkW,paramW;
  boolean errflag;
  byte inbuf[10];                //hold responses
  byte numTI[10];                //data in ti format
  double mantissa,temp;          //to get he mantissa
  char value[20],stripped[20];
  byte namelength;

  namelength=len(sname);
  numtoTIst86(num,numTI);
  //now to send
  worked=false;   
  errflag=false;                      //in case we err
#ifdef DEBUG
  Serial.println("Sending");
#endif
  sendByte(comp);                     //machine ID 
  sendByte(0xC9);                     //RTS
  sendByte(0x0C);                     //length of command-LSB
  sendByte(0x00);                     //length of command-MSB
  sendByte(0x0A);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  sendByte(0x00);                     //ID for real number
  sendByte(namelength);               //length of var name
  for (i=0;i<namelength;i++)sendByte(sname[i]);                     //ASCII of variable name (A-Z)

  for(i=0;i<(8-namelength);i++) sendByte(0x20);    //padding and dud bytes
 
  chkW=0x0A+namelength;               //create checksum
  for(i=0;i<namelength;i++) chkW=chkW+(word)sname[i];
  for(i=0;i<(8-namelength);i++) chkW=chkW+0x0020;
  sendByte(lowByte(chkW));            //send LSB
  sendByte(highByte(chkW));           //send MSB
  delay(100);
  for(i=0;i<4;i++) {                  //get response
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=ACK){
    for(i=0;i<4;i++){
#ifdef DEBUG
      Serial.print(inbuf[i]);
      Serial.print(" ");
#endif
    }
#ifdef DEBUG
    Serial.println(" received. ACK not received");
#endif
    errflag=true;
    goto err;
  }
  for(i=0;i<4;i++) {                  //get response
    inbuf[i]=0;
    inbuf[i]=getByte();
  }
  if(inbuf[1]!=CTS){
#ifdef DEBUG
    Serial.println("CTS not received");
#endif
    errflag=true;
    goto err;
  }
   
#ifdef DEBUG
  Serial.println("Sending ACK");
#endif
  delay(100);
  sendACK();
  sendByte(comp);                     //machine ID
  sendByte(0x15);                     //DATA  
  sendByte(0x0A);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB

  chkW = 0;                           //send the value and
  for (j = 0;j<10;j++){       //create checksum simultaneously
    sendByte(numTI[j]);
    chkW = chkW + numTI[j];  
  }

  sendByte(lowByte(chkW));            //send checksum in parts
  sendByte(highByte(chkW));

  for(i=0;i<4;i++) {
    inbuf[i]=0;
    inbuf[i]=getByte();               //response from TI
  }
    if(inbuf[1]!=ACK){
#ifdef DEBUG
      Serial.println("ACK not received");
#endif
      errflag=true;
      goto err;
  }

  sendByte(comp);                     //machine ID
  sendByte(0x92);                     //EOT
  sendByte(0x00);                     //size of data-LSB
  sendByte(0x00);                     //size of data-MSB
  
err:worked=!errflag;
  delay(100);                        //give calc time to recover
  return worked;
}//end sendNum86

void ArtoTI::sendOP(char *ops){
  //send the operator. not an exhaustive list. if it's not here, send the code with sendCmd 
  //now sendOP is a 2 char function
  word outkey;
  char key,key2;
  outkey=0;
  key=toupper(ops[0]);
  key2=toupper(ops[1]);
  if(comp==0x06){              //86
#ifdef DEBUG
    Serial.println(ops);
#endif
    switch (key){
      case '+': sendANkey(' ');
                sendCmd(0x0C);
                sendANkey(' ');
                break;
      case '-': sendANkey(' ');
                sendCmd(0x0C);
                sendANkey(' ');
                break;
      case '*': sendANkey(' ');
                sendCmd(0x0E);
                sendANkey(' ');
                break;
      case '/': sendANkey(' ');
                sendCmd(0x0F);
                sendANkey(' ');
                break;
      case '(': sendCmd(0x11);
                break;
      case ')': sendCmd(0x12);
                break;
      case '^': sendCmd(0x10);
                break;
      case '=': sendCmd(0x15);
                break;
      case ',': sendCmd(0x18);
                break;
      case '.': sendCmd(0x1B);
                break;
      case 'A': switch (key2){
                  case 'C': sendOP("(");  //special key Acos
                            sendCmd(0x63);
                            break;
                  case 'S': sendOP("(");  //special key square Asin
                            sendCmd(0x61);
                            break;
                  case 'T': sendOP("("); //Atan
                            sendCmd(0x65);
                            break;
                }
                break;
      case 'C': if(key2=='H') {
                  sendCmd(0x1A);  //special case change sign
                } else if(key2=='S'){  //special key cos
                  sendOP("(");
                  sendCmd(0x62);
                }                  
                break;
      case 'D': if(key2=='N'){      //down arrow
                  sendCmd(0x04);
                }
                break;
      case 'E': if(key2=='E'){
                  sendCmd(0x26);  //special case the EE key
                }else if(key2=='X'){
                  sendCmd(0x69);  //e^
                }else if(key2=='T'){
                  sendCmd(0x07);	//86 exit
                }
                break;
      case 'L': if(key2=='G'){
                  sendCmd(0x6A);  //special case log
                  sendOP("(");
                }else if(key2=='N'){    //ln
                  sendCmd(0x68);
                  sendOP("(");
                }else if(key2=='F'){  //left
                  sendCmd(0x02);
                }
                break;
      case 'P': if(key2=='I') sendCmd(0x5E);  //special key Pi
                break;
      case 'R': if(key2=='T') sendCmd(0x01);  //right
                break;
      case 'S': switch (key2){
                  case 'T': sendCmd(0x16);   //special key store
                            break;
                  case 'R': sendCmd(0x67);  //special key square root
                            sendOP("(");
                            break;
                  case 'Q': sendCmd(0x66);  //squared
                            break;
                  case 'N': sendOP("(");  //sin
                            sendCmd(0x60);
                            break;
                }
                break;
      case 'T': if(key2=='N') {
                  sendOP("(");
                  sendCmd(0x64); //tan
                  break;
      case 'U': if(key2 =='P'){  //up arrow
                  sendCmd(0x03);
                }
                break;
                }
    }
  }else if(comp==0x23){        //83+/84+
#ifdef DEBUG
    Serial.println(ops);
#endif
    switch (key){
      case '+': sendCmd(0x80);
                break;
      case '-': sendCmd(0x81);
                break;
      case '*': sendCmd(0x82);
                break;
      case '/': sendCmd(0x83);
                break;
      case '(': sendCmd(0x85);
                break;
      case ')': sendCmd(0x86);
                break;
      case '^': sendCmd(0x84);
                break;
      case '=': outkey=0x00;  //not in 83/84+
                break;
      case ',': sendCmd(0x8B);
                break;
      case '.': sendCmd(0x8D);
                break;
      case '%': sendCmd(0xFB0C);    //quote
                break;
      case '{': sendCmd(0xEC);
                break;
      case '}': sendCmd(0xED);
                break;
      case 'A': switch (key2){
                  case 'C': sendCmd(0xBA);  //acos
                            break;
                  case 'S': sendCmd(0xB8);  //asin
                            break;
                  case 'T': sendCmd(0xBC);  //atan
                            break;
                }
                break;
      case 'C': if(key2=='H'){
                  sendCmd(0x8C);  //special case change sign
                } else if(key2=='S'){ 
                  sendCmd(0xB9);  //cos
                }
                break;
      case 'D': if(key2=='N'){      //down arrow
                  sendCmd(0x04);
                }
                break;
      case 'E': if(key2=='E'){
                  sendCmd(0x98);  //special case the EE key
                }else if(key2=='X'){    //e^
                  sendCmd(0xC0);
                }
                break;
      case 'L': if(key2=='G'){
                  sendCmd(0xC1);  //special case log
                }else if(key2=='N'){    //ln
                  sendCmd(0xBF);
                }else if(key2=='F'){  //left
                  sendCmd(0x02);
                }
                break;
      case 'P': if(key2=='I') outkey=0xB5;  //special key Pi
                sendCmd(outkey);
                break;
      case 'R': if(key2=='T') sendCmd(0x01);  //right
                break;
      case 'S': switch(key2){
                  case 'T': sendCmd(0x8A);  //special key store
                            break;
                  case 'R': sendCmd(0xBE);  //special key square root
                            if(calctype==84) sendOP("(");
                            break;
                  case 'Q': sendCmd(0xBD);  //squared
                            break;
                  case 'N': sendCmd(0xB7);  //sin
                            break;
                }
                break;
      case 'T': if(key2=='N') sendCmd(0xBB); //tan
                break;
      case 'U': if(key2 =='P'){  //up arrow
                  sendCmd(0x03);
                }
                break;
    }
  }
}
  
byte ArtoTI::Start(byte tippin,byte ringpin){
  byte temp;  
  Data0=tippin;
  Data1=ringpin;
  resetlines();
  temp=getStatus();
  if(temp==0){
    Serial.println("Unrecognized calculator");
  }
  resetlines();
  return temp;
}
