/*
	ArtoTI.h - Library for communicating between an Arduino and 
	TI 83+, 84+, or 86
	Created by William Massano, February 14, 2014
	Released to the public domain after magazine publication
*/
#ifndef ArtoTI_h
#define ArtoTI_h

#include <Arduino.h>

class ArtoTI
{
	public:
		byte comp;								//holds code for which calculator we are talking to
		int calctype;							//holds 83 for 83+, 84 for 84+, 82 for 82, 86 for 86 and -83 for an 83
		byte Data0;                  			//synonym for tippin
		byte Data1;                 			//synonym for ringpin
		ArtoTI(byte tippin,byte ringpin);								//constructor
		void clearScreen();
		void Enter();							//the "Enter" key on calculator
		void floatout(double num,int dplaces);	//this corrects a flaw in the Arduino Serial.print
		void getDir();                			//get a directory of variables mostly to test comm
		double getNum(char *sname);      		//for now getnum will return a value  
		byte getStatus();						//returns which calculator is connected
		void Plot(double *xlist, double *ylist, int numpoints, byte mark); //plot y vs x using mark 0=box, 1=plus, 2=period
		void sendANstring(char *str);			//send a string of characters A-Z, 0-9 also (,), and comma
		void sendByte(byte paramB);				//send a byte value
		void sendCmd(word paramW);				//send a calculator command code
		void sendList(byte listnum, double *list,int listlen);	//send a list to  a list var
		boolean sendNum(char *sname,double num);	//send a number num to a calculator variable defined in sname string
												//the string is A-Z on 83/84+ and and string of characters for the 86 must be null terminated
		void sendOP(char *ops);					//send an operator public, but best if used by sendANstring
	private:
		void clearlines();						//for debugging only
		byte getByte();
		boolean getByteTimed(byte* paramB);
		void getDir86();              			//get for 86 a directory of variables mostly to test comm
		double getNum86(char *sname);    		//TI86 specific routine
		byte len(char *str);					//like strlen
		char* ltrim(char *string, char junk);	//trim leading spaces
		void numtoTIst83(double num, byte *numTI);  //numTI will have 10 elements
		void numtoTIst86(double num, byte *numTI);  //numTI will have 10 elements
		void Plot86(double *xlist, double *ylist, int numpoints, byte mark); //plot y vs x using mark 0=box, 1=plus, 2=period
		void resetlines();						//set all lines to input mode with pullup
		void sendACK();							//send acknowledge
		void sendANkey(char key);				//send a character
		void sendList86(byte listnum, double *list,int listlen);
		boolean sendNum86(char *sname,double num);	//internal for 86
		byte Start(byte tippin,byte ringpin);	//initialize and store calculator code in comp

#define ACK 0x56
#define VHEADER 0x06
#define CTS 0x09
#define DATA 0x15
#define EOT 0x92
#define ASCII0 0x30          //ASCII code for zero
//#define DEBUG                //for debugging
			
};
#endif

